@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2011, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: writeout.c,v 1.32 2008-04-30 21:20:09 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -45,6 +44,7 @@ typedef enum {
   VAR_TOTAL_TIME,
   VAR_NAMELOOKUP_TIME,
   VAR_CONNECT_TIME,
+  VAR_APPCONNECT_TIME,
   VAR_PRETRANSFER_TIME,
   VAR_STARTTRANSFER_TIME,
   VAR_SIZE_DOWNLOAD,
@@ -62,6 +62,7 @@ typedef enum {
   VAR_REDIRECT_COUNT,
   VAR_FTP_ENTRY_PATH,
   VAR_REDIRECT_URL,
+  VAR_SSL_VERIFY_RESULT,
   VAR_NUM_OF_VARS /* must be the last */
 } replaceid;
 
@@ -79,6 +80,7 @@ static const struct variable replacements[]={
   {"time_total", VAR_TOTAL_TIME},
   {"time_namelookup", VAR_NAMELOOKUP_TIME},
   {"time_connect", VAR_CONNECT_TIME},
+  {"time_appconnect", VAR_APPCONNECT_TIME},
   {"time_pretransfer", VAR_PRETRANSFER_TIME},
   {"time_starttransfer", VAR_STARTTRANSFER_TIME},
   {"size_header", VAR_HEADER_SIZE},
@@ -93,6 +95,7 @@ static const struct variable replacements[]={
   {"num_redirects", VAR_REDIRECT_COUNT},
   {"ftp_entry_path", VAR_FTP_ENTRY_PATH},
   {"redirect_url", VAR_REDIRECT_URL},
+  {"ssl_verify_result", VAR_SSL_VERIFY_RESULT},
   {NULL, VAR_NONE}
 };
 
@@ -104,7 +107,7 @@ void ourWriteOut(CURL *curl, const char *writeinfo)
   long longinfo;
   double doubleinfo;
 
-  while(*ptr) {
+  while(ptr && *ptr) {
     if('%' == *ptr) {
       if('%' == ptr[1]) {
         /* an escaped %-letter */
@@ -116,7 +119,7 @@ void ourWriteOut(CURL *curl, const char *writeinfo)
         char *end;
         char keepit;
         int i;
-        if(('{' == ptr[1]) && (end=strchr(ptr, '}'))) {
+        if(('{' == ptr[1]) && ((end = strchr(ptr, '}')) != NULL)) {
           bool match = FALSE;
           ptr+=2; /* pass the % and the { */
           keepit=*end;
@@ -164,7 +167,8 @@ void ourWriteOut(CURL *curl, const char *writeinfo)
                 break;
               case VAR_REDIRECT_TIME:
                 if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_REDIRECT_TIME, &doubleinfo))
+                   curl_easy_getinfo(curl, CURLINFO_REDIRECT_TIME,
+                                     &doubleinfo))
                   fprintf(stream, "%.3f", doubleinfo);
                 break;
               case VAR_TOTAL_TIME:
@@ -183,14 +187,22 @@ void ourWriteOut(CURL *curl, const char *writeinfo)
                    curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME, &doubleinfo))
                   fprintf(stream, "%.3f", doubleinfo);
                 break;
+              case VAR_APPCONNECT_TIME:
+                if(CURLE_OK ==
+                   curl_easy_getinfo(curl, CURLINFO_APPCONNECT_TIME,
+                                     &doubleinfo))
+                  fprintf(stream, "%.3f", doubleinfo);
+                break;
               case VAR_PRETRANSFER_TIME:
                 if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_PRETRANSFER_TIME, &doubleinfo))
+                   curl_easy_getinfo(curl, CURLINFO_PRETRANSFER_TIME,
+                                     &doubleinfo))
                   fprintf(stream, "%.3f", doubleinfo);
                 break;
               case VAR_STARTTRANSFER_TIME:
                 if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_STARTTRANSFER_TIME, &doubleinfo))
+                   curl_easy_getinfo(curl, CURLINFO_STARTTRANSFER_TIME,
+                                     &doubleinfo))
                   fprintf(stream, "%.3f", doubleinfo);
                 break;
               case VAR_SIZE_UPLOAD:
@@ -200,12 +212,14 @@ void ourWriteOut(CURL *curl, const char *writeinfo)
                 break;
               case VAR_SIZE_DOWNLOAD:
                 if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD, &doubleinfo))
+                   curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD,
+                                     &doubleinfo))
                   fprintf(stream, "%.0f", doubleinfo);
                 break;
               case VAR_SPEED_DOWNLOAD:
                 if(CURLE_OK ==
-                   curl_easy_getinfo(curl, CURLINFO_SPEED_DOWNLOAD, &doubleinfo))
+                   curl_easy_getinfo(curl, CURLINFO_SPEED_DOWNLOAD,
+                                     &doubleinfo))
                   fprintf(stream, "%.3f", doubleinfo);
                 break;
               case VAR_SPEED_UPLOAD:
@@ -231,13 +245,19 @@ void ourWriteOut(CURL *curl, const char *writeinfo)
                    && stringp)
                   fputs(stringp, stream);
                 break;
+              case VAR_SSL_VERIFY_RESULT:
+                if(CURLE_OK ==
+                   curl_easy_getinfo(curl, CURLINFO_SSL_VERIFYRESULT,
+                                     &longinfo))
+                  fprintf(stream, "%ld", longinfo);
+                break;
               default:
                 break;
               }
               break;
             }
           }
-          if(!match) {
+          if(FALSE == match) {
             fprintf(stderr, "curl: unknown --write-out variable: '%s'\n", ptr);
           }
           ptr=end+1; /* pass the end */
